@@ -6,16 +6,43 @@ This registry helps us find providers during runtime. We can also apply some
 operations to all or several providers at the same time.
 '''
 
+class RegistryException(Exception):
+    pass
 
 class Registry:
-    '''This registry collects all vocab providers.
+    '''
+    This registry collects all skos providers.
     '''
 
     def __init__(self):
-        self.providers = []
+        self.providers = {}
 
     def register_provider(self, provider):
-        self.providers.append(provider)
+        if provider.get_vocabulary_id() in self.providers:
+            raise RegistryException(
+                'A provider with this id has already been registered.')
+        self.providers[provider.get_vocabulary_id()] = provider
+
+    def remove_provider(self, id):
+        '''
+        Remove the provider with the given id.
+        
+        Returns the provider or False if the id is unknown.
+        '''
+        if id in self.providers:
+            p = self.providers.get(id, False)
+            del self.providers[id]
+            return p
+        else:
+            return False
+
+    def get_provider(self, id):
+        '''
+        Get a provider by id.
+
+        Returns the provider or False if the id is unknown.
+        '''
+        return self.providers.get(id, False)
 
     def get_providers(self, **kwargs):
         '''Get all providers registered.
@@ -23,10 +50,10 @@ class Registry:
         If keyword ids is present, get only the providers with this id.
         '''
         if not 'ids' in kwargs:
-            return self.providers
+            return list(self.providers.values())
         else:
-            return [p for p in self.providers
-                    if p.get_vocabulary_id() in kwargs['ids']]
+            return [self.providers[k] for k in self.providers.keys() 
+                    if k in kwargs['ids']]
 
     def find(self, query, **kwargs):
         '''Launch a query across all or a selection of providers.
@@ -50,4 +77,4 @@ class Registry:
         Returns a list of dicts. Each dict has two keys: id and concepts.
         '''
         return [{'id': p.get_vocabulary_id(), 'concepts': p.get_all()}
-                for p in self.providers]
+                for p in self.providers.values()]
